-- Main.lua

if not chud_accentchars then rawset(_G, "chud_accentchars", {}) end
chud_accentchars.silver = SKINCOLOR_SKY
chud_accentchars.shadow = SKINCOLOR_RED
chud_accentchars.blaze = SKINCOLOR_KETCHUP
chud_accentchars.iclyn = SKINCOLOR_ICY
chud_accentchars.milne = SKINCOLOR_PURPLE

rawset(_G, "CHUD", {
    leftofs = 0,
    topofs = 1,
    bottomofs = 0,
    rightofs = 0,
    prevtic = 0,
    fade = 0,
    width = 128,
    bgcolor = 30,
    bgcolor2 = 28,
    accentmap = V_YELLOWMAP,
    -- Exists for debugging purposes dofiwejfmweiofwejm
    forceredtime = false,
    slideSpeed = 3,
    boss = nil,
    prevhealth = 0,
    dmgtime = 0,
    lasthealth = 0,
    -- Accent color override if the player is playing as that character and is using their prefcolor
    accentchars = chud_accentchars,
    bossdist = 8192*FRACUNIT,
    enabled = true,
})

if bossnames == nil then rawset(_G, "bossnames", {}) end
bossnames[MT_EGGMOBILE] = "Egg Mobile"
bossnames[MT_EGGMOBILE2] = "Egg Slimer"
bossnames[MT_EGGMOBILE3] = "Sea Egg"
bossnames[MT_EGGMOBILE4] = "Egg Colosseum"
bossnames[MT_FANG] = "Fang"
bossnames[MT_METALSONIC_BATTLE] = "Metal Sonic"
bossnames[MT_BLACKEGGMAN] = "Brak Eggman"
bossnames[MT_CYBRAKDEMON] = "Brak Eggman"
COM_AddCommand("hud_color", function(p, color)
    if not color then
        CONS_Printf(p, "hud_color <color>")
        CONS_Printf(p, "Changes the HUD accent color, set to None to reset")
        return
    end
    local col = R_GetColorByName(color)
    if (col == SKINCOLOR_NONE) then 
        p.accentoverride = nil
        CONS_Printf(p, "Accent color reset")
        return
    end
    p.accentoverride = col
    CONS_Printf(p, "Accent color set to " + R_GetNameByColor(col))
end)
CV_RegisterVar({
    name = "hud_enabled",
    defaultValue = 1,
    flags = CV_CALL,
    PossibleValue = CV_OnOff,
    func = function(var)
        if string.lower(var.string) == "on" or var.string == "1" then
            CHUD.enabled = true
        elseif string.lower(var.string) == "off" or var.string == "0" then
            CHUD.enabled = false
            hud.enable("rings")
            hud.enable("time")
            hud.enable("score")
            hud.enable("lives")
        end
    end
})
-- Returns the accent skincolor that should be used, going from:
-- Accent color override (chud_color)
-- Super color
-- Character accent color
-- Player skincolor
CHUD.GetAccentNoValidation = function(p)
    if p.chud then
        if p.accentoverride then return p.accentoverride end
    end
    if not p.mo then return p.skincolor end
    if p.skincolor == skins[p.mo.skin].prefcolor and CHUD.accentchars[p.mo.skin] and not CHUD.IsSuper(p) then
        -- Special cases
        if p.mo.skin == "blaze" and p.blazeflamecolor then return p.blazeflamecolor end
        if p.mo.skin == "silver" and silv_auracolor then return silv_auracolor(p) end
        if p.mo.skin == "shadow" and p.shadow_skatecolor then return p.shadow_skatecolor end
        return CHUD.accentchars[p.mo.skin]
    end
    return p.mo.color
end
-- Returns the accent skincolor but with extra validation
-- ...Except there is no validation
CHUD.GetAccent = function(p)
    return CHUD.GetAccentNoValidation(p) or SKINCOLOR_WHITE
end
-- Draws a color ramp, because repeating code is pain
CHUD.DrawColorRamp = function(color, v, x, y, w, h, f, reverse)
    local ramp = skincolors[color].ramp
    v.drawFill(x, y, w, h, ramp[15]|f)
    local perEl = (FRACUNIT/16) * w
    if reverse then
        local reali = 0
        for i = 15, 0, -1 do
            v.drawFill(x + (reali * perEl)/FRACUNIT, y, perEl/FRACUNIT, h, ramp[i]|f)
            reali = $ + 1
        end
    else
        for i = 0, 15, 1 do
            v.drawFill(x + (i * perEl)/FRACUNIT, y, perEl/FRACUNIT, h, ramp[i]|f)
        end
    end
end

CHUD.DrawBarElement = function(el, value, v, x, y, w, h, f, color)
    local ramp = skincolors[color].ramp

    local thin = 3
    if el.borderSize then thin = el.borderSize end

    v.drawFill(x, y, w, h, (el.bg or CHUD.bgcolor)|f)
    v.drawFill(x + thin, y + thin, w - thin*2, h - thin*2, CHUD.bgcolor2|f)
    v.drawFill(x + thin, y + thin, FixedMul(max(min(FixedDiv(value, el.max), FRACUNIT), 0), (w-thin*2)*FRACUNIT)/FRACUNIT, h - thin*2, ramp[5]|f)
end
CHUD.DrawElement = function(p, v, el, x, y, w, h, f)
    el.draw(p, v, x, y, w, h, f, el)
    if el.rampThickness then
        CHUD.DrawColorRamp(el.rampcolor or CHUD.GetAccent(p), v, x, y + h - el.rampThickness, w, el.rampThickness, f, el.reverseRamp)
    end
end

CHUD.AddElement = function(type, el)
    table.insert(type, el)
    return el
end

-- Re-runs the custom support code
CHUD.RedoCustomSupport = function()
    -- Silver's bar drawing function exists and silver support hasn't been run yet
    if silv_drawBarHUD and not CHUD.silverSupport then

        -- Prevent it from running
        rawset(_G, "silv_barHUD", false)
        CHUD.silverSupport = true
        -- Add a custom R - T - S element for it instead
        table.insert(CHUD.RTSElements, {
            loffset = 0,
            roffset = 0,
            height = 7,
            isVisible = function(p) return p.mo and p.mo.skin == "silver" and p.mo.silv_energy < SILVER_MAX_ENERGY end,
            rampThickness = 1,
            borderSize = 2,
            visanim = true,
            draw = function(p, v, x, y, w, h, f, el)
                el.max = SILVER_MAX_ENERGY
                local col = SKINCOLOR_SKY
                if silv_auracolor then col = silv_auracolor(p) end
                local e = p.mo.silv_energy or 0
                if e < SILVER_MAX_ENERGY/4 and leveltime % 8 < 4 then
                    col = SKINCOLOR_RED
                end
                if p.powers[pw_super] then col = p.mo.color end
                el.rampcolor = col
                CHUD.DrawBarElement(el, e, v, x, y, w, h, f, col)
            end
        })
    end
end


addHook("MapLoad", function()
    CHUD.RedoCustomSupport()
end)

-- Returns true if the player is super or in a sol form
CHUD.IsSuper = function(p)
    return p.powers[pw_super] or (p.solchar and p.solchar.istransformed)
end

-- Returns true if the player is in a hyper form (Crystal/Alt Sonic, Jelly Tails). XMomentum hyper forms not included
CHUD.IsHyper = function(p)
    if not p.mo then return false end
    return ((p.mo.skin == "altsonic" or p.mo.skin == "crystalsonic") and p.althyper and p.althyper.transforming)
    or (p.mo.skin == "jellytails" and (p.jellyhyper or p.hyperTransform))
end

addHook("PlayerThink", function(p)
    if not p.chud then
        p.chud = {
            nosuper = 0,
            redblink = 0,
            greenblink = 0,
            prevrings = 0,
            supertime = 0,
            nohyper = 0,
            hypertime = 0,
        }
    end
    if not CHUD.IsHyper(p) then
        p.chud.nohyper = $ + 1
        p.chud.hypertime = 0
    else
        p.chud.nohyper = 0
        p.chud.hypertime = $ + 1
    end
    if not CHUD.IsSuper(p) then
        p.chud.nosuper = $ + 1
        p.chud.supertime = 0
        p.chud.solcancel = false
    else
        p.chud.nosuper = 0
        p.chud.supertime = $ + 1
        if p.CancelSolTransformAnimation then p.chud.solcancel = true end
    end
end)
addHook("PostThinkFrame", function()
    for p in players.iterate() do
        if not p.chud then continue end
        if p.chud.redblink then p.chud.redblink = $ - 1 end
        if p.chud.greenblink then p.chud.greenblink = $ - 1 end
        if p.chud.prevrings > p.rings then p.chud.redblink = 10 end
        if p.chud.prevrings < p.rings then p.chud.greenblink = 10 end
        p.chud.prevrings = p.rings
        if CHUD.boss and CHUD.boss.valid then
            if CHUD.dmgtime > 0 then CHUD.dmgtime = $ - 1 end
            if CHUD.prevhealth ~= CHUD.boss.health then
                CHUD.lasthealth = CHUD.prevhealth
                if CHUD.prevhealth > CHUD.boss.health then CHUD.dmgtime = 10 end
                CHUD.prevhealth = CHUD.boss.health
            end
        end
    end
end)

hud.add(function(v, p)
    if not CHUD.enabled then return end
    hud.disable("rings")
    hud.disable("time")
    hud.disable("score")
    hud.disable("lives")
    if not p.chud then return end
    if not p.mo.color then return end
    if leveltime < TICRATE*2 then
        CHUD.fade = 0
    end
    local time = TICRATE
    if p.solchar and p.solchar.istransformed then
        -- Has to be longer because sol transformations have a long ass animation
        time = TICRATE*3 + TICRATE/10
        -- ...Except they don't always have to be
        if p.chud.solcancel then
            time = TICRATE
        end
    end
    local curtarget = FRACUNIT
    -- Ha ha yes
    if (p.chud.supertime and p.chud.supertime < time) or (not p.chud.supertime and p.chud.nosuper < TICRATE)
    or (p.chud.hypertime and p.chud.hypertime < time) or (not p.chud.hypertime and p.chud.nohyper < TICRATE)
    or p.playerstate == PST_DEAD or p.exiting or maptol & TOL_NIGHTS or G_IsSpecialStage(gamemap) then
        curtarget = 0
    end
    CHUD.DrawRTS(v, p)
    CHUD.DrawLives(v, p)
    CHUD.DrawSpeedometer(v, p)
    if CHUD.fade ~= curtarget and CHUD.prevtic ~= leveltime then
        local spd = (abs(curtarget - CHUD.fade)/TICRATE + FRACUNIT/TICRATE/8)*6
        CHUD.fade = $ + min(max(curtarget - CHUD.fade, -spd), spd)
    end
    CHUD.prevtic = leveltime
end, "game")
