-- Rings-Time-Score.lua
CHUD.DrawLabeledRTSElement = function(el, value, v, x, y, w, h, f, labelf, valuef)
    local available = (w - v.stringWidth(value)) - 8
    local tw = v.stringWidth(el.label)
    v.drawString(x + 1 + el.loffset, y + 1, el.label, f|(labelf or 0), (tw >= available and "thin") or "left")
    v.drawString(x + w - (el.roffset + 1), y + 1, value, f|(valuef or CHUD.accentmap), "right")
end

local transflags = {
    V_90TRANS,
    V_80TRANS,
    V_70TRANS,
    V_60TRANS,
    V_50TRANS,
    V_40TRANS,
    V_30TRANS,
    V_20TRANS,
    V_10TRANS,
    0,
}

addHook("BossThinker", function(mo)
    CHUD.boss = mo
end)

local l = #transflags
for i = l, 1, -1 do
    table.insert(transflags, transflags[i])
end 

local colors = {31, 47, 46, 45, 44, 43, 42, 41, 40, 39, 38, 37, 36, 35}

CHUD.SetTheme = function(light)
    if light then
        CHUD.bgcolor = 0
        CHUD.bgcolor2 = 14
        colors = {35, 34, 34, 34, 33, 33, 32, 210, 210, 209, 209, 209, 208, 208}
    else
        CHUD.bgcolor = 30
        CHUD.bgcolor2 = 28
        colors = {31, 47, 46, 45, 44, 43, 42, 41, 40, 39, 38, 37, 36, 35}
    end
    local l = #colors
    for i = l, 1, -1 do
        table.insert(colors, colors[i])
    end

end

CHUD.SetTheme(false)

COM_AddCommand("hud_theme", function(p, theme)
    if theme and string.lower(theme) == "dark" then
        CHUD.SetTheme(false)
        print("Theme set to dark")
    elseif theme and string.lower(theme) == "light" then
        CHUD.SetTheme(true)
        print("Theme set to light")
    else
        print("chud_theme <dark | light>")
        print("Sets your hud theme")
    end
end, COM_LOCAL)


CHUD.RTSElements = {
    {
        padtop = 0,
        loffset = 0,
        roffset = 0,
        height = 6,
        borderSize = 2,
        max = TICRATE*30,
        isVisible = function(p) return (p.powers[pw_underwater] and p.powers[pw_underwater] < underwatertics) or (p.powers[pw_spacetime] and p.powers[pw_spacetime] < spacetimetics) end,
        rampThickness = 1,
        visanim = true,
        draw = function(p, v, x, y, w, h, f, el)
            if CHUD.RingWarning(p) then
                el.bg = colors[(leveltime % #colors) + 1]
            else
                el.bg = nil
            end
            if p.mo.eflags & MFE_UNDERWATER then el.max = underwatertics else
                el.max = spacetimetics end
            CHUD.DrawBarElement(el, p.powers[pw_spacetime] or p.powers[pw_underwater], v, x, y, w, h, f, SKINCOLOR_AQUA)
        end
    },
    {
        padtop = 1,
        label = "TIME",
        loffset = 6,
        roffset = 0,
        height = 10,
        isVisible = function() return true end,
        draw = function(p, v, x, y, w, h, f, el)
            local t = p.realtime
            if mapheaderinfo[gamemap] and mapheaderinfo[gamemap].countdown then
                t = CHUD.TimeLeft()
            end
            --local prefix = ""
            --if mapheaderinfo[gamemap] and mapheaderinfo[gamemap].countdown then
            --    t = mapheaderinfo[gamemap].countdown*TICRATE - leveltime
            --    if leveltime % 16 < 8 then
            --        prefix = "\x85"
            --    end
            --end
            --el.label = prefix .. "TIME"
            local timestr = string.format("%02d", G_TicsToMinutes(t, true)) .. ":" ..
            string.format("%02d", G_TicsToSeconds(t)) .. "." ..
            string.format("%02d", G_TicsToCentiseconds(t))
            CHUD.DrawLabeledRTSElement(el, timestr, v, x, y, w, h, f)
            if mapheaderinfo[gamemap] and mapheaderinfo[gamemap].countdown or CHUD.forceredtime then
                t = mapheaderinfo[gamemap].countdown*TICRATE - leveltime
                local tflag = transflags[(leveltime % #colors)*FRACUNIT/#colors*#transflags/FRACUNIT + 1]
                CHUD.DrawLabeledRTSElement(el, timestr, v, x, y, w, h, (f & ~V_ALPHAMASK)|tflag|V_REDMAP)
            end
        end
    },
    {
        label = "SCORE",
        loffset = 6,
        roffset = 0,
        height = 10,
        isVisible = function() return true end,
        draw = function(p, v, x, y, w, h, f, el)
            CHUD.DrawLabeledRTSElement(el, tostring(p.score), v, x, y, w, h, f)
        end
    },
    {
        padtop = 5,
        loffset = 0,
        roffset = 0,
        height = 10,
        label = "BOSS",
        isVisible = function(p) return CHUD.boss and CHUD.boss.valid and CHUD.boss.health and not (p.mo and P_AproxDistance(CHUD.boss.x - p.mo.x, CHUD.boss.y - p.mo.y) > CHUD.bossdist) end,
        visanim = true,
        width = CHUD.width/8*9,
        draw = function(p, v, x, y, w, h, f, el)
            el.label = bossnames[CHUD.boss and CHUD.boss.valid and CHUD.boss.type] or "BOSS"
            CHUD.DrawLabeledRTSElement(el, CHUD.boss.health*100/CHUD.boss.info.spawnhealth + "%", v, x, y, w, h, f, V_REDMAP, V_REDMAP)
        end
    },
    {
        loffset = 0,
        roffset = 0,
        height = 10,
        isVisible = function(p) return CHUD.boss and CHUD.boss.valid and CHUD.boss.health and not (p.mo and P_AproxDistance(CHUD.boss.x - p.mo.x, CHUD.boss.y - p.mo.y) > CHUD.bossdist) end,
        visanim = true,
        padbottom = 5,
        width = CHUD.width/8*9,
        draw = function(p, v, x, y, w, h, f, el)
            local health = 0
            local maxh = 1
            local color = SKINCOLOR_RED

            if CHUD.boss and CHUD.boss.valid then
                health = CHUD.boss.health
                maxh = CHUD.boss.info.spawnhealth
                if CHUD.dmgtime then
                    if leveltime % 6 <= 3 then
                        color = SKINCOLOR_WHITE
                        -- health = health + ((CHUD.lasthealth*10 - health)*CHUD.dmgtime)/10
                        health = CHUD.lasthealth
                    end
                end
            end
            el.max = maxh
            CHUD.DrawBarElement(el, health, v, x, y + (CHUD.dmgtime and CHUD.dmgtime % 4 - 2), w, h, f, color)
        end
    }
}
CHUD.TimeLeft = function()
    if mapheaderinfo[gamemap] and mapheaderinfo[gamemap].countdown then
        return mapheaderinfo[gamemap].countdown*TICRATE - min(leveltime, mapheaderinfo[gamemap].countdown*TICRATE)
    end
    return 2147483647
end
CHUD.RingWarning = function(p)
    return p.forcewarning or (not CHUD.IsSuper(p) and p.rings <= 0) or (CHUD.IsSuper(p) and p.rings <= 10) or CHUD.TimeLeft() < 10*TICRATE
end
CHUD.DrawRTS = function(v, p)
    local fadefactor = CHUD.width + CHUD.leftofs
    local x = -fadefactor + FixedMul((fadefactor + CHUD.leftofs)*FRACUNIT, CHUD.fade)/FRACUNIT
    local y = CHUD.topofs
    local f = V_SNAPTOTOP|V_SNAPTOLEFT
    local col = CHUD.bgcolor
    if CHUD.RingWarning(p) then
        col = colors[(leveltime % #colors) + 1]
    end
    v.drawFill(x, y, CHUD.width, 10, col|f)
    v.drawString(x + 4, y + 1, "RINGS", f|CHUD.accentmap)
    v.drawString(x + CHUD.width - 1, y + 1, p.rings, f|CHUD.accentmap, "right")

    local blinkmap = 0
    if p.chud.greenblink then blinkmap = V_GREENMAP|min(max(V_10TRANS*(10 - p.chud.greenblink), 0), V_90TRANS) end
    if p.chud.redblink then blinkmap = V_REDMAP|min(max(V_10TRANS*(10 - p.chud.redblink), 0), V_90TRANS) end
    if blinkmap then
        v.drawString(x + CHUD.width - 1, y + 1, p.rings, f|blinkmap, "right")
    end
    if not ((p.powers[pw_underwater] and p.powers[pw_underwater] < underwatertics) or (p.powers[pw_spacetime] and p.powers[pw_spacetime] < spacetimetics)) then
        CHUD.DrawColorRamp(CHUD.GetAccent(p), v, x, y + 10, CHUD.width, 1, f)
    end
    y = $ + 10
    for _,el in pairs(CHUD.RTSElements) do
        local visible = el.isVisible(p, el, v)
        if not visible and not el.visanim then continue end
        if not el.tempx then el.tempx = x end
        local elw = el.width or CHUD.width
        if el.visanim then
            if leveltime ~= CHUD.prevtic then
                local spd = CHUD.slideSpeed
                local e = (fadefactor*elw)/CHUD.width
                if visible then
                    spd = $ + abs(x - el.tempx)/8
                    el.tempx = $ + max(min(x - el.tempx, spd), -spd)
                else
                    spd = $ + abs(-fadefactor - el.tempx)/8
                    el.tempx = $ + max(min(-e - el.tempx, spd), -spd)
                end
            end
            if el.tempx <= -CHUD.width then continue end
        else
            el.tempx = x
        end
        if not el.tempy then el.tempy = y end
        y = $ + (el.padtop or 0)
        if leveltime ~= CHUD.prevtic then el.tempy = $ + max(min(y - el.tempy, 1), -1) end
        CHUD.DrawElement(p, v, el, el.tempx, el.tempy, elw, el.height, f)
        if el.tempx == x then y = $ + el.height + (el.padbottom or 0) end
    end
end
